/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.type;

import com.vividsolutions.jts.geom.Geometry;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.FieldManager;
import nl.cloudfarming.client.geoviewer.SplitGeometryHandler;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.Node;
import org.openide.util.Lookup;

/**
 * Split handler for {@link Field}
 * @author johan
 */
public class FieldSplitHandler extends SplitGeometryHandler {
    private static final Logger LOG = Logger.getLogger("nl.cloudfarming.client.field.type");
    private final Field sourceField;

    public FieldSplitHandler(Field field) {
        this.sourceField = field;
    }

    @Override
    public Geometry getSource() {
        return sourceField.getGeometry();
    }

    @Override
    public boolean handle(Geometry[] geometries, Node sourceNode) {
        FieldManager mgr = Lookup.getDefault().lookup(FieldManager.class);
        Project project  = EnterpriseIdProvider.ProjectFinder.find(sourceField.getURI());
        FileObject fieldsFolder = FieldManagerImpl.getFieldsFolder(project, true);

        for (int i = 0; i < geometries.length; i++) {
            // create new field based on original field and new geometry,
            Field newField = new Field(sourceField);
            newField.setGeometry(geometries[i]);
            newField.setName(sourceField.getName() + "-part-" + i);
            
            mgr.save(newField);
            
            FileObject fieldFile = fieldsFolder.getFileObject(newField.getItemIdNumber(), Field.EXT);
            try {
                DataObject dobj = DataObject.find(fieldFile);
                addToMap(dobj.getNodeDelegate());
            } catch (DataObjectNotFoundException ex) {
                LOG.log(Level.WARNING, "could not find dataobject for file {0}: {1}", new Object[]{fieldFile, ex});
            }
        }
        // expire original field:
        sourceField.setEndDate(new Date());
        removeFromMap(sourceNode);
        
        return true;
    }
       
}
