/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.type;

import java.awt.datatransfer.Transferable;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.swing.Action;
import nl.cloudfarming.client.field.model.TreatmentZone;
import nl.cloudfarming.client.geoviewer.SingleObjectLayer;
import nl.cloudfarming.client.util.actions.OpenAction;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.util.Lookup;
import org.openide.util.Utilities;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;

/**
 * Node to display a {@link TreatmentZone}
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
final class TreatmentZoneNode extends AbstractNode implements PropertyChangeListener, Serializable {

    private TreatmentZone treatmentZone;
    /**
     * Constructs a new node for a {@link TreatmentZone}
     *
     * @param treatmentZone the {@link TreatmentZone} to create this node for
     */
    public TreatmentZoneNode(TreatmentZone treatmentZone) {
        super(Children.LEAF, Lookups.fixed(treatmentZone, new OnOffPalette(treatmentZone),new SingleObjectLayer(treatmentZone.getName(), treatmentZone, Lookup.EMPTY), new TreatmentZoneSplitHandler(treatmentZone)));
        assert treatmentZone != null : "Should be a TreatmentZone in the lookup of a TreatmentZonedNode";
        this.treatmentZone = treatmentZone;
        setName(treatmentZone.getName());
        setIconBaseWithExtension("nl/cloudfarming/client/field/type/treatmentzone.png");
        treatmentZone.addPropertyChangeListener(TreatmentZone.PROP_NAME, this);
        
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        return null;
    }

    @Override
    public Action[] getActions(boolean context) {
        List<Action> actions = new ArrayList<>();
        actions.add(new OpenAction(TreatmentZone.MIMETYPE, this));
        //add all actions registered in the layer file
        for (Action action : Utilities.actionsForPath("Loaders/application/x-agrosense-treatmentzone/Actions")) {
            actions.add(action);
        }
        
        return actions.toArray(new Action[actions.size()]);
    }

    @Override
    public boolean canRename() {
        return true;
    }

    @Override
    public void setName(String s) {
        super.setName(s);
        treatmentZone.setName(s);
    }

    /**
     * The display name should be the name of the {@link TreatmentZone#name} passed in the constructor
     * @return 
     */
    @Override
    public String getDisplayName() {
        return treatmentZone.getName();
    }
    
    
    
    @Override
    public void propertyChange(PropertyChangeEvent pce) {
        if (pce.getNewValue() != null) {
            this.setName(pce.getNewValue().toString());
        }
    }
}
