/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.type.wizard;

import nl.cloudfarming.client.geoviewer.render.GeometricalsPreviewPanel;
import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;
import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.FieldManager;
import nl.cloudfarming.client.geoviewer.Geometrical;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ui.OpenProjects;
import org.openide.awt.Mnemonics;
import org.openide.util.Lookup;
import org.openide.util.NbBundle.Messages;

/**
 * 
 * @author Maciek
 */
@Messages({
    "GeometricalTaskObjectSelectVisualPanel_label_text_select_farm=Select farm:",
    "GeometricalTaskObjectSelectVisualPanel_label_text_select_field=Select field:",
    "GeometricalTaskObjectSelectVisualPanel_label_text_selection_preview=Selection preview:",
    "GeometricalTaskObjectSelectVisualPanel_wizard_step_name_field=Select fields",
    "GeometricalTaskObjectSelectVisualPanel_wizard_step_name_treatmentzone=Select treatment zones",
    "GeometricalTaskObjectSelectVisualPanel_wizard_step_field_header=Select fields",
    "GeometricalTaskObjectSelectVisualPanel_wizard_step_treatmentzone_header=Select fields",
})
public final class GeometricalSelectPanel extends JPanel implements ListSelectionListener {

    private Project selectedProject;
    private List<Geometrical> selectedGeometricals;
    private Class desiredClass;
    private String headerLabel;
    private String stepName;
    private JList fieldList;
    private JLabel labelSelectFarm;
    private JLabel labelSelectField;
    private JLabel labelSelectionPreview;
    private JScrollPane fieldScrollPane;
    private JScrollPane farmScrollPane;
    private JPanel geometryPreviewPanel;
    private JList farmList;
    private int selectionMode;
    
    /**
     * Creates new form GeometricalTaskObjectSelectVisualPanel
     */
    GeometricalSelectPanel(Class desiredClass, int selectionMode) {
        this.desiredClass = desiredClass;
        this.selectionMode = selectionMode;
        assignBundledValues();
        initComponents();
        
        fieldList.addListSelectionListener(this);
        farmList.addListSelectionListener(this);
        fillProjectList();
        fillGeometryList();
    }

    private void assignBundledValues() {
        if (desiredClass == Field.class) {
            headerLabel = Bundle.GeometricalTaskObjectSelectVisualPanel_wizard_step_field_header();
            stepName = Bundle.GeometricalTaskObjectSelectVisualPanel_wizard_step_name_field();
        } else {
            headerLabel = Bundle.GeometricalTaskObjectSelectVisualPanel_wizard_step_treatmentzone_header();
            stepName = Bundle.GeometricalTaskObjectSelectVisualPanel_wizard_step_name_treatmentzone();
        }
    }

    private void fillProjectList() {
        farmList.setListData(OpenProjects.getDefault().getOpenProjects());
    }

    private void fillGeometryList() {
        selectedProject = (Project) farmList.getSelectedValue();
        List<? extends Geometrical> geometricals = new ArrayList<>();
        FieldManager fieldManager = Lookup.getDefault().lookup(FieldManager.class);
        if (fieldManager != null && selectedProject != null) {
            if (desiredClass == Field.class) {
                geometricals = fieldManager.getFields(selectedProject);
            } 
//            else {
//                geometricals = fieldManager.getTreatmentZones(selectedProject);
//            }
        }
        fieldList.setListData(geometricals.toArray());
    }

    public List<Geometrical> getSelectedGeometricals() {
        if (selectedGeometricals != null) {
            return selectedGeometricals;
        } else {
            return null;
        }
    }

    public Project getSelectedProject() {
        return selectedProject;
    }

    @Override
    public String getName() {
        return stepName;
    }
    
    public JList getGeometricalsList() {
        return fieldList;
    }
    
    public JList getProjectList() {
        return farmList;
    }

    /**
     * This method is called from within the constructor to initialize the form.
     */
    private void initComponents() {
        // Create Labels
        labelSelectFarm = new JLabel();
        labelSelectField = new JLabel();
        labelSelectField.setText(headerLabel);
        labelSelectionPreview = new JLabel();
        
        Mnemonics.setLocalizedText(labelSelectFarm, Bundle.GeometricalTaskObjectSelectVisualPanel_label_text_select_farm()); // NOI18N
        Mnemonics.setLocalizedText(labelSelectField, Bundle.GeometricalTaskObjectSelectVisualPanel_label_text_select_field()); // NOI18N
        Mnemonics.setLocalizedText(labelSelectionPreview, Bundle.GeometricalTaskObjectSelectVisualPanel_label_text_selection_preview()); // NOI18N

        // Project list
        farmScrollPane = new JScrollPane();
        farmList = new JList();
        farmList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        farmScrollPane.setViewportView(farmList);
        
        // Field list
        fieldScrollPane = new JScrollPane();
        fieldList = new JList();
        fieldList.setSelectionMode(selectionMode);
        fieldScrollPane.setViewportView(fieldList);

        // Empty geometrypanel
        geometryPreviewPanel = new JPanel();
        GroupLayout previewPanelLayout = new GroupLayout(geometryPreviewPanel);
        geometryPreviewPanel.setLayout(previewPanelLayout);
        previewPanelLayout.setHorizontalGroup(
            previewPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGap(0, 0, Short.MAX_VALUE)
        );
        previewPanelLayout.setVerticalGroup(
            previewPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGap(0, 0, Short.MAX_VALUE)
        );

        // Construct visual
        GroupLayout layout = new GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
                    .addComponent(labelSelectFarm)
                    .addComponent(farmScrollPane, GroupLayout.DEFAULT_SIZE, 142, Short.MAX_VALUE))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
                    .addComponent(fieldScrollPane, GroupLayout.DEFAULT_SIZE, 141, Short.MAX_VALUE)
                    .addComponent(labelSelectField))
                .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(36, 36, 36)
                        .addComponent(labelSelectionPreview)
                        .addGap(0, 127, Short.MAX_VALUE))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(18, 18, 18)
                        .addComponent(geometryPreviewPanel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(GroupLayout.Alignment.LEADING)
            .addGroup(GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addGap(82, 82, 82)
                .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
                    .addGroup(GroupLayout.Alignment.TRAILING, layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
                        .addComponent(labelSelectFarm)
                        .addComponent(labelSelectionPreview))
                    .addComponent(labelSelectField, GroupLayout.Alignment.TRAILING))
                .addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(GroupLayout.Alignment.TRAILING)
                    .addComponent(geometryPreviewPanel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(fieldScrollPane, GroupLayout.Alignment.LEADING, GroupLayout.DEFAULT_SIZE, 246, Short.MAX_VALUE)
                    .addComponent(farmScrollPane, GroupLayout.Alignment.LEADING))
                .addContainerGap())
        );
    }

    @Override
    public void valueChanged(ListSelectionEvent event) {
        if (selectedProjectChanged(event)) {
            selectedProject = (Project) farmList.getSelectedValue();
            fillGeometryList();
        } else {
            showGeometricalPreview();
        }
    }

    private boolean selectedProjectChanged(ListSelectionEvent event) {
        return (event.getSource() == farmList);
    }

    private List<Geometrical> removeObjectsWithoutGeometry(List<Geometrical> dirtyGeometricals) {
        List<Geometrical> cleanGeometricals = new ArrayList<>();
        for (Geometrical geometrical : dirtyGeometricals) {
            if (geometrical.getGeometry() != null) {
                cleanGeometricals.add(geometrical);
            }
        }
        return cleanGeometricals;
    }

    private void showGeometricalPreview() {
        selectedGeometricals = removeObjectsWithoutGeometry(fieldList.getSelectedValuesList());
        if (selectedGeometricals.isEmpty()) {
            geometryPreviewPanel.setVisible(false);
        } else {
            geometryPreviewPanel.setLayout(new BorderLayout());
            geometryPreviewPanel.removeAll();
            geometryPreviewPanel.add(new GeometricalsPreviewPanel(selectedGeometricals));
            geometryPreviewPanel.validate();
            geometryPreviewPanel.repaint();
            geometryPreviewPanel.setVisible(true);
        }
    }
}
